function DraconisErus_Raid_HasPlayer(player)
	local retVal = false
	local raiders = DraconisErus_Raid_GetRaiders()
	local index = DraconisErus_TableSearch(raiders,player)
	if index > 0 then
		retVal = true
	end
	return retVal
end

function DraconisErus_Raid_GetRaiders()
	local raiders = {}
	for i = 1, GetNumGroupMembers() do
		local name, rank = GetRaidRosterInfo(i)
		if name ~= "" and name ~= nil then
			name = DraconisErus_FixName(name)
			if DraconisErus_Guild_HasPlayer(name) then
				table.insert(raiders,name)
			end
		end
	end
	table.sort(raiders)
	return raiders
end

function DraconisErus_Raid_GetLeader()
	local retVal = ""
	for i = 1, GetNumGroupMembers() do
		local name, rank = GetRaidRosterInfo(i)
		if name ~= "" and name ~= nil then
			if rank == 2 then
				name = DraconisErus_FixName(name)
				retVal = name
				break
			end
		end
	end

	return retVal
end

function DraconisErus_Raid_GetRaidInfo()
	local officers = {}
	local nonofficers = {}
	local raidinfo = {}
	local raidleader = ""

	for i = 1, GetNumGroupMembers() do
		local rname, rrank, _, _, _, rclass, rzone, ronline, risDead = GetRaidRosterInfo(i)
		if rname ~= "" and rname ~= nil then
			local rpoints = 0
			if DEDKPPoints[DERR_REALM][DERR_GUILD][rname] ~= nil then rpoints = DEDKPPoints[DERR_REALM][DERR_GUILD][rname] end
			if rrank == 2 then
				raidinfo.raidleader = rname
				raidinfo[rname] = { name = rname, rank = rrank, zone = rzone, online = ronline, isDead = risDead, class = rclass, points = rpoints }
				DraconisErus_Debug("raidinfo:" .. rname)
			elseif DraconisErus_Guild_IsOfficer(rname) then
				table.insert(officers,rname)
				raidinfo[rname] = { name = rname, rank = rrank, zone = rzone, online = ronline, isDead = risDead, class = rclass, points = rpoints }
				DraconisErus_Debug("raidinfo:" .. rname)
			elseif DraconisErus_Guild_HasPlayer(rname) then
				table.insert(nonofficers,rname)
				raidinfo[rname] = { name = rname, rank = rrank, zone = rzone, online = ronline, isDead = risDead, class = rclass, points = rpoints }
				DraconisErus_Debug("raidinfo:" .. rname)
			else
				DraconisErus_Debug("raidinfo:ignore:"..rname)
			end
			DraconisErus_Queue_Remove(rname)
		end
	end

	table.sort(officers)
	table.sort(nonofficers)

	raidinfo.officers = officers
	raidinfo.nonofficers = nonofficers
	
	return raidinfo
end

function DraconisErus_Raid_GetAllPlayers(inQueue)
	local allPlayers = {}
	for index, player in pairs(DraconisErus_Raid_GetRaiders()) do
		if DraconisErus_TableSearch(allPlayers,player) == 0 then
			table.insert(allPlayers,player)
		end
	end
	if inQueue then
		for index, player in pairs(DEDKPSettings["RaidQueue"]) do
			if DraconisErus_TableSearch(allPlayers,player) == 0 then
				table.insert(allPlayers,player)
			end
		end
	end

	return allPlayers
end

function DraconisErus_Raid_Sync()
	if IsInRaid() then
		DraconisErus_Debug("Starting raid sync...")
		DERR_WAIT_FOR_RAID = true
		local broadcaster = DraconisErus_Raid_FindBroadcaster()
		if broadcaster ~= "" then
			DraconisErus_Comm_Send("DERRSENDRAID", "", "WHISPER", broadcaster)
		end
	end
end

function DraconisErus_Raid_CanBroadcast(playerinfo)
	local retVal = false
	if DraconisErus_TableSearch(DEDKPSettings["AddonUsers"],playerinfo.name) > 0 then
		if playerinfo.name ~= DERR_PLAYER and playerinfo.online then
			retVal = true
		else
			DraconisErus_Debug("player:"..playerinfo.name.." cannot broadcast.")
		end
	else
		DraconisErus_Debug("player:"..playerinfo.name.." not an addon user.")
	end
	return retVal
end

function DraconisErus_Raid_GetPoints()
	local retVal = {}
	for index, value in pairs(DEDKPPoints[DERR_REALM][DERR_GUILD]) do
		if DraconisErus_SenderAuth(index,1,0,1) then
			retVal[index] = value
		end
	end
	return retVal
end

function DraconisErus_Raid_FindBroadcaster()
	local broadcaster = ""
	local raidinfo = DraconisErus_Raid_GetRaidInfo()
	local zonetext = GetZoneText()

	local playerinfo = raidinfo[raidinfo.raidleader]
	--check if raid leader can broadcast
	if DraconisErus_Raid_CanBroadcast(playerinfo) then
		broadcaster = playerinfo.name
	end

	--check if guild officers can broadcast
	if broadcaster == "" then
		DraconisErus_Debug("officercount:"..table.getn(raidinfo.officers))
		for _, player in pairs(raidinfo.officers) do
			playerinfo = raidinfo[player]
			if DraconisErus_Raid_CanBroadcast(playerinfo) then
				broadcaster = player
				break
			end
		end
	end

	--check if any remaining players can broadcast
	if broadcaster == "" then
		DraconisErus_Debug("nonofficercount:"..table.getn(raidinfo.nonofficers))
		for _, player in pairs(raidinfo.nonofficers) do
			playerinfo = raidinfo[player]
			if DraconisErus_Raid_CanBroadcast(playerinfo) then
				broadcaster = player
				break
			end
		end
	end

	DraconisErus_Debug("broadcaster:"..broadcaster)
	return broadcaster

end
